// nRF24L01 2.4GHz Scanner
// 2016-11-26  T. Nakagawa

#include "MsTimer2.h"
#include "NRF24L01.h"
#include "U8glib.h"

#define NRF24L01_PIN_CE 10
#define NRF24L01_PIN_CS 9
#define CHANNELS 126
#define LCD_PIN_CS 4
#define LCD_PIN_A0 3
#define LCD_CONTRAST 200
#define SW_PIN 2
#define SW_INT 0
#define FIXED_POINT_BIT 5
#define SIGNAL_SCALE_BIT 8

NRF24L01 radio(NRF24L01_PIN_CE, NRF24L01_PIN_CS);
U8GLIB_MINI12864 u8g(LCD_PIN_CS, LCD_PIN_A0);
uint16_t signal_strength[CHANNELS];  // Signal strength (0~256 * 16).
uint8_t mode = 0;

void updateSignalStrength() {
  for (uint8_t i = 0; i < CHANNELS; i++) {
    // Measure the signal strength.
    radio.SetChannel(i);
    radio.Enable();
    delayMicroseconds(170);  // Wait for Tstby2a+Tdelay_AGC.
    radio.Disable();
    // Update the signal strength.
    // NewValue = lambda * CurrentValue + (1 - lambda) * OldValue, where lambda = 1/32.
    // All values are represented as fixed-point numbers multiplied by 32.
    // Therefore, NewValue' = CurrentValue' / 32 + (32 - 1) * OldValue' / 32.
    // CurrentValue is 256 (2^SIGNAL_SCALE_BIT) if a signal is sensed or 0 otherwise.
    const uint16_t &old_val = signal_strength[i];
    const uint16_t cur_val = ((uint16_t)radio.TestRPD() << SIGNAL_SCALE_BIT);
    signal_strength[i] = cur_val + (((old_val << FIXED_POINT_BIT) - old_val) >> FIXED_POINT_BIT);
  }
}

void drawGraph() {
  u8g.drawVLine(0, 0, 64);
  u8g.drawVLine(127, 0, 64);
  for (uint8_t i = 0; i < CHANNELS; i++) {
    u8g.drawVLine(1 + i, 64 - (signal_strength[i] >> FIXED_POINT_BIT), 64);
  }
}

void drawList() {
  u8g.setFont(u8g_font_micro);  // 4x5 font.
  const uint8_t base = (mode - 1) * 42;
  uint8_t idx = 0;
  for (uint8_t i = 0; i < 11; i++) {
    for (uint8_t j = 0; j < 4; j++) {
      char buf[9];
      sprintf(buf, "%03d:%3d ", base + idx, (signal_strength[base + idx] >> FIXED_POINT_BIT));
      u8g.drawStr(j * 8 * 4, 5 + i * 6, buf);
      idx++;
      if (idx >= 42) break;
    }
  }
}

void changeMode() {
  static uint32_t last_changed = 0;

  if (millis() > last_changed + 100) {
    mode++;
    if (mode >= 4) mode = 0;
    last_changed = millis();
  }
}

void setup() {
  u8g.setContrast(LCD_CONTRAST);
  u8g.setColorIndex(1);

  MsTimer2::set(100, updateSignalStrength);

  pinMode(SW_PIN, INPUT_PULLUP);
  attachInterrupt(SW_INT, changeMode, FALLING);
}

void loop() {
  MsTimer2::stop();
  u8g.firstPage();
  do {
    if (mode == 0) {
      drawGraph();
    } else {
      drawList();
    }
  } while (u8g.nextPage());
  MsTimer2::start();
  delay(250);
}

